<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

namespace assignsubmission_dta\privacy;

use core_privacy\local\metadata\collection;
use core_privacy\local\request\writer;
use core_privacy\local\request\contextlist;
use mod_assign\privacy\assign_plugin_request_data;

/**
 * provider for data privacy
 *
 * @package assignsubmission_dta
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @copyright Gero Lueckemeyer and student project teams
 */
class provider implements \core_privacy\local\metadata\provider,
        \mod_assign\privacy\assignsubmission_provider,
        \mod_assign\privacy\assignsubmission_user_provider {

    /**
     * File area for dta submission assignment.
     */
    const ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION = "submissions_dta";

    /**
     * This is the metadata information for the assignsubmission_dta plugin.
     * @param collection $collection the collection of metadata filled by the plugin
     */
    public static function get_metadata(collection $collection): collection {
        $collection->add_subsystem_link(
            'core_files',
            [],
            'privacy:metadata:core_files'
        );

        $collection->add_database_table(
            'assignsubmission_dta_summary',
            [
                'assignment_id' => 'privacy:metadata:assignsubmission_dta_summary:assignmentid',
                'submission_id' => 'privacy:metadata:assignsubmission_dta_summary:submissionid',
                'timestamp' => 'privacy:metadata:assignsubmission_dta_summary:timestamp',
                'global_stacktrace' => 'privacy:metadata:assignsubmission_dta_summary:global_stacktrace',
                'successful_competencies' => 'privacy:metadata:assignsubmission_dta_summary:successful_competencies',
                'tested_competencies' => 'privacy:metadata:assignsubmission_dta_summary:tested_competencies',

            ],
            'privacy:metadata:assignsubmission_dta_summary'
        );

        $collection->add_database_table(
            'assignsubmission_dta_result',
            [
                'assignment_id' => 'privacy:metadata:assignsubmission_dta_summary:assignmentid',
                'submission_id' => 'privacy:metadata:assignsubmission_dta_summary:submissionid',
                'package_name' => 'privacy:metadata:assignsubmission_dta_result:package_name',
                'class_name' => 'privacy:metadata:assignsubmission_dta_result:class_name',
                'name' => 'privacy:metadata:assignsubmission_dta_result:name',
                'state' => 'privacy:metadata:assignsubmission_dta_result:state',
                'failure_type' => 'privacy:metadata:assignsubmission_dta_result:failure_type',
                'failure_reason' => 'privacy:metadata:assignsubmission_dta_result:failure_reason',
                'stacktrace' => 'privacy:metadata:assignsubmission_dta_result:stacktrace',
                'column_number' => 'privacy:metadata:assignsubmission_dta_result:column_number',
                'line_number' => 'privacy:metadata:assignsubmission_dta_result:line_number',
                'position' => 'privacy:metadata:assignsubmission_dta_result:position',
            ],
            'privacy:metadata:assignsubmission_dta_result'
        );

        $collection->add_database_table(
            'assignsubmission_dta_recommendations',
            [
                'assignment_id' => 'privacy:metadata:assignsubmission_dta_summary:assignmentid',
                'submission_id' => 'privacy:metadata:assignsubmission_dta_summary:submissionid',
                'topic'=> 'privacy:metadata:assignsubmission_dta_recommendations:topic',
                'exercise_name'=> 'privacy:metadata:assignsubmission_dta_recommendations:exercise_name',
                'url'=> 'privacy:metadata:assignsubmission_dta_recommendations:url',
                'difficulty'=> 'privacy:metadata:assignsubmission_dta_recommendations:difficulty',
                'score'=> 'privacy:metadata:assignsubmission_dta_recommendations:score',
            ],
            'privacy:metadata:assignsubmission_dta_recommendations'
        );

        $collection->add_external_location_link('dta_backend', [
                'assignment_id' => 'privacy:metadata:assignsubmission_dta_summary:assignmentid',
                'submission_id' => 'privacy:metadata:assignsubmission_dta_summary:submissionid',
                'submissioncontent' => 'privacy:metadata:core_files',
            ],
            'privacy:metadata:dta_backend'
        );

        return $collection;
    }

    /**
     * This is covered by mod_assign provider and the query on assign_submissions.
     *
     * @param  int $userid The user ID that we are finding contexts for.
     * @param  contextlist $contextlist A context list to add sql and params to for contexts.
     */
    public static function get_context_for_userid_within_submission(int $userid, contextlist $contextlist) {
        // This is already fetched from mod_assign.
    }

    /**
     * This is also covered by the mod_assign provider and its queries.
     *
     * @param  \mod_assign\privacy\useridlist $useridlist An object for obtaining user IDs of students.
     */
    public static function get_student_user_ids(\mod_assign\privacy\useridlist $useridlist) {
        // This is already fetched from mod_assign.
    }

    /**
     * If you have tables that contain userids and you can generate entries in your tables without creating an
     * entry in the assign_submission table then please fill in this method.
     *
     * @param  userlist $userlist The userlist object
     */
    public static function get_userids_from_context(\core_privacy\local\request\userlist $userlist) {
        // Not required.
    }

    /**
     * Export all user data for this plugin.
     *
     * @param  assign_plugin_request_data $exportdata Data used to determine which context and user to export and other useful
     * information to help with exporting.
     */
    public static function export_submission_user_data(assign_plugin_request_data $exportdata) {
        // We currently don't show submissions to teachers when exporting their data.
        $context = $exportdata->get_context();
        if ($exportdata->get_user() != null) {
            return null;
        }
        $user = new \stdClass();
        $assign = $exportdata->get_assign();
        $plugin = $assign->get_plugin_by_type('assignsubmission', 'file');
        $submission = $exportdata->get_pluginobject();
        $files = $plugin->get_files($exportdata->get_pluginobject(), $user);
        foreach ($files as $file) {
            $userid = $exportdata->get_pluginobject()->userid;
            $dtaresultsummary = DBUtils::getresultsummaryfromdatabase($assign->id, $submission->id);
            // Submitted file.
            writer::with_context($exportdata->get_context())->export_file($exportdata->get_subcontext(), $file)
            // DTA result.
            ->export_related_data($dtaresultsummary);
        }
    }

    /**
     * Any call to this method should delete all user data for the context defined in the deletion_criteria.
     *
     * @param  assign_plugin_request_data $requestdata Information useful for deleting user data.
     */
    public static function delete_submission_for_context(assign_plugin_request_data $requestdata) {
        global $DB;

        $fs = get_file_storage();
        $fs->delete_area_files($requestdata->get_context()->id, 'assignsubmission_dta', \assignsubmission_dta\privacy\provider::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION);

        $assignmentid = $requestdata->get_assign()->get_instance()->id;

        // Delete records from assignsubmission_dta tables.
        $DB->delete_records('assignsubmission_dta_result', ['assignment_id' => $assignmentid]);
        $DB->delete_records('assignsubmission_dta_summary', ['assignment_id' => $assignmentid]);
        $DB->delete_records('assignsubmission_dta_recommendations', ['assignment_id' => $assignmentid]);
    }

    /**
     * A call to this method should delete user data (where practical) using the userid and submission.
     *
     * @param  assign_plugin_request_data $deletedata Details about the user and context to focus the deletion.
     */
    public static function delete_submission_for_userid(assign_plugin_request_data $deletedata) {
        global $DB;

        $assignmentid = $deletedata->get_assign()->get_instance()->id;
        $submissionid = $deletedata->get_pluginobject()->id;

        $fs = get_file_storage();
        $fs->delete_area_files($deletedata->get_context()->id, 'assignsubmission_dta', \assignsubmission_dta\privacy\provider::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION,
                $submissionid);

        // Delete records from assignsubmission_dta tables. Also possible with a list as below.
        $DB->delete_records('assignsubmission_dta_result', [
            'assignment_id' => $assignmentid,
            'submission_id' => $submissionid,
            ]);
        $DB->delete_records('assignsubmission_dta_summary', [
            'assignment_id' => $assignmentid,
            'submission_id' => $submissionid,
            ]);
        $DB->delete_records('assignsubmission_dta_recommendations', [
            'assignment_id' => $assignmentid,
            'submission_id' => $submissionid,
            ]);
    }

    /**
     * Deletes all submissions for the submission ids / userids provided in a context.
     * assign_plugin_request_data contains:
     * - context
     * - assign object
     * - submission ids (pluginids)
     * - user ids
     * @param  assign_plugin_request_data $deletedata A class that contains the relevant information required for deletion.
     */
    public static function delete_submissions(assign_plugin_request_data $deletedata) {
        global $DB;

        if (empty($deletedata->get_submissionids())) {
            return;
        }
        $fs = get_file_storage();
        list($sql, $params) = $DB->get_in_or_equal($deletedata->get_submissionids(), SQL_PARAMS_NAMED);
        $fs->delete_area_files_select($deletedata->get_context()->id, 'assignsubmission_file', ASSIGNSUBMISSION_FILE_FILEAREA_SUBMISSION,
                $sql, $params);

        $params['assignid'] = $deletedata->get_assignid();
        $DB->delete_records_select('assignsubmission_dta_result', "assignment_id = :assignid AND submissionid $sql", $params);
        $DB->delete_records_select('assignsubmission_dta_summary', "assignment_id = :assignid AND submissionid $sql", $params);
        $DB->delete_records_select('assignsubmission_dta_recommendations', "assignment_id = :assignid AND submissionid $sql", $params);
    }
}
